__version__ = "0.1"
__git_commiter_name__ = "Tal Yalon"
__git_commiter_email__ = "tal.yalon@gmail.com"
__git_branch__ = '(Not currently on any branch)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '68e16f621134bcc2393d2ab8b0fd84297c832e05'
__git_head_subject__ = 'Finished release v0.1.'
__git_head_message__ = ''
__git_dirty_diff__ = ''


SETUP_INFO = dict(
    name = 'infi.tracing',
    version = '0.1',
    author = 'Tal Yalon',
    author_email = 'tal.yalon@gmail.com',

    url = 'https://infinigit.infinidat.com/host/tracing',
    license = 'PSF',
    description = """short description here""",
    long_description = """long description here""",

    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Python Software Foundation License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = ['distribute', 'greenlet', 'cython', 'infi.pyutils'],
    setup_requires = ['Cython'],
    namespace_packages = ['infi'],

    package_dir = {'': 'src'},
    package_data = {'': []},
    include_package_data = True,
    zip_safe = False,

    entry_points = dict(
        console_scripts = [],
        gui_scripts = [],
        ),

)

if SETUP_INFO['url'] is None:
    _ = SETUP_INFO.pop('url')

def build_ext_modules():
    from setuptools.extension import Extension

    return [Extension("infi.tracing.ctracing",
               language="c++",
               sources=["src/infi/tracing/ctracing.pyx", "src/infi/tracing/thread_storage.cpp", 
                        "src/infi/tracing/trace_dump.cpp"],
               include_dirs=["src/infi/tracing"],
               define_macros=[("_REENTRANT", 1)],
               libraries=["stdc++"],
               extra_compile_args=["-std=c++11", "-Wno-format-security"],
               extra_link_args=["-std=c++11"])]

def setup():
    from setuptools import setup as _setup
    from setuptools import find_packages
    try:
        from Cython.Distutils import build_ext
        SETUP_INFO['ext_modules'] = build_ext_modules()
        SETUP_INFO['cmdclass'] = {'build_ext': build_ext}
    except ImportError:
        import sys
        sys.stderr.write("cython not found, install before running setup.py.\n")

    SETUP_INFO['packages'] = find_packages('src')
    _setup(**SETUP_INFO)
 
if __name__ == '__main__':
    setup()



__import__("pkg_resources").declare_namespace(__name__)
import sys
from infi.pyutils.contexts import contextmanager
from infi.pyutils.decorators import wraps

__all__ = ['set_tracing', 'unset_tracing', 'set_func_cache_size', 'suspend_tracing', 'resume_tracing',
           'no_tracing_context_recursive', 'no_tracing_recursive',
           'tracing_output_to_syslog', 'tracing_output_to_file', 'tracing_output_to_stdout', 'tracing_output_to_stderr',
           'NO_TRACE', 'NO_TRACE_NESTED', 'TRACE_FUNC_NAME', 'TRACE_FUNC_PRIMITIVES', 'TRACE_FUNC_REPR']

# Same as in ctracing:
NO_TRACE              = 0
NO_TRACE_NESTED       = 1
TRACE_FUNC_NAME       = 2
TRACE_FUNC_PRIMITIVES = 3
TRACE_FUNC_REPR       = 4


def _filter_all(*args, **kwargs):
    return True


def tracing_output_to_syslog(ident, facility):
    from infi.tracing.ctracing import ctracing_set_output_to_syslog
    ctracing_set_output_to_syslog(ident, facility)


def tracing_output_to_file(path):
    from infi.tracing.ctracing import ctracing_set_output_to_file
    ctracing_set_output_to_file(path)


def tracing_output_to_stdout():
    from infi.tracing.ctracing import ctracing_set_output_to_stdout
    ctracing_set_output_to_stdout()


def tracing_output_to_stderr():
    from infi.tracing.ctracing import ctracing_set_output_to_stderr
    ctracing_set_output_to_stderr()


def set_tracing(filter_func=_filter_all):
    from infi.tracing.ctracing import ctracing_set_profile, ctracing_start_trace_dump
    ctracing_start_trace_dump()
    ctracing_set_profile(filter_func)


def unset_tracing():
    from infi.tracing.ctracing import ctracing_stop_trace_dump
    sys.setprofile(None)
    ctracing_stop_trace_dump()


def set_func_cache_size(size):
    """Sets the function LRU cache size. The cache is used to determine whether to trace a function or not to trace it,
    and which level of tracing should be done."""
    from infi.tracing.ctracing import ctracing_set_func_cache_size
    ctracing_set_func_cache_size(size)


def suspend_tracing():
    from infi.tracing.ctracing import suspend
    suspend()


def resume_tracing():
    from infi.tracing.ctracing import resume
    resume()


@contextmanager
def no_tracing_context_recursive():
    try:
        suspend_tracing()
        yield
    finally:
        resume_tracing()

def no_tracing_recursive(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        with no_tracing_context_recursive():
            return func(*args, **kwargs)
    return wrapper
